package dev.kske.chess.piece;

import dev.kske.chess.Board;
import dev.kske.chess.Move;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Pawn.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class Pawn extends Piece {

	public Pawn(Color color, Board board) {
		super(color, board);
	}

	@Override
	public boolean isValidMove(Move move) {
		// TODO: en passant
		boolean	step		= move.isVertical() && move.yDist == 1;
		boolean	doubleStep	= move.isVertical() && move.yDist == 2;
		boolean	strafe		= move.isDiagonal() && move.xDist == 1;
		if (getColor() == Color.WHITE) doubleStep &= move.pos.y == 6;
		else doubleStep &= move.pos.y == 1;
		return (step ^ doubleStep ^ strafe) && move.ySign == (getColor() == Color.WHITE ? -1 : 1) && isFreePath(move);
	}

	@Override
	protected boolean isFreePath(Move move) {
		// Two steps forward
		if (move.yDist == 2)
			return board.getBoardArr()[move.pos.x][move.dest.y - move.ySign] == null && checkDestination(move);
		// One step forward
		else if (move.xDist == 0) return board.getDest(move) == null;
		// Capture move
		else return board.getDest(move) != null && board.getDest(move).getColor() != getColor();
	}

	@Override
	public Type getType() { return Type.PAWN; }
}
