package dev.kske.chess.piece;

import dev.kske.chess.Board;
import dev.kske.chess.Move;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>Piece.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public abstract class Piece {

	protected Color	color;
	protected Board	board;

	public Piece(Color color, Board board) {
		this.color	= color;
		this.board	= board;
	}

	public abstract boolean isValidMove(Move move);

	protected boolean isFreePath(Move move) {
		// Only check destination by default
		return checkDestination(move);
	}

	/**
	 * Checks if the destination of a move is empty or a piece from the opposing
	 * team
	 * 
	 * @param move The move to check
	 * @return {@code false} if the move's destination is from the same team
	 */
	protected final boolean checkDestination(Move move) {
		return board.getDest(move) == null || board.getDest(move).getColor() != getColor();
	}

	public abstract Type getType();

	public Color getColor() { return color; }

	public static enum Type {
		KING, QUEEN, ROOK, KNIGHT, BISHOP, PAWN;
	}

	public static enum Color {
		WHITE, BLACK;
	}
}
